//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//  $Id:$
//
//  Color notehead plugin
//	Noteheads are colored according to pitch. User can change to color by
//  modifying the colors array. First element is C, second C# etc...
//
//  Copyright (C)2008 Werner Schweer and others
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

//
// This is ECMAScript code (ECMA-262 aka "Java Script")
//

//---------------------------------------------------------
//    init
//    this function will be called on startup of mscore
//---------------------------------------------------------

function init() {
	//print("Repitch plugin started.");
}

//-------------------------------------------------------------------
//    run
//    this function will be called when activating the
//    plugin menu entry
//
//    global Variables:
//    pluginPath - contains the plugin path; file separator is "/"
//-------------------------------------------------------------------

function run()
{
	// simply copied from other plugins, 
	// not touched, should work
	
    var loader = new QUiLoader(null);
    var file   = new QFile(pluginPath + "/repitch.ui");
    file.open(QIODevice.OpenMode(QIODevice.ReadOnly, QIODevice.Text));
    form = loader.load(file, null);
    form.buttonBox.accepted.connect(accept);

    form.show();
}

//---------------------------------------------------------
//    accept
//    called when user presses "Accept" button
//---------------------------------------------------------
function accept()
{
	// read parameters from the form
	// source cursor (pitches source) parameters
	var source_staff = form.group_source.spin_source_staff.value;
	var source_voice = form.group_source.spin_source_voice.value;
	var source_measure = form.group_source.spin_source_measure.value;			// measure offset
	var source_note_offset = form.group_source.spin_source_note_offset.value;	

	// target cursor (voice to repitch) parameters 		  
	var target_staff = form.group_target.spin_target_staff.value;
	var target_voice = form.group_target.spin_target_staff.value;
	var target_measure_start = form.group_target.spin_target_measure_start.value;
	var target_measure_stop = form.group_target.spin_target_measure_stop.value;
	var target_note_offset = form.group_target.spin_target_note_offset.value;
	
	// adjust one based gui-values to zero based
	source_staff -= 1;
	source_voice -= 1;
	source_measure -= 1;
	source_note_offset -= 1;
	target_staff -= 1;
	target_voice -= 1;
	target_measure_start -= 1;
	target_measure_stop -= 1;
	target_note_offset -= 1;
	
	curScore.startUndo();
	
	// initialize source cursor
	
    var source_cursor   = new Cursor(curScore);
    source_cursor.staff = source_staff;
    source_cursor.voice = source_voice;
    source_cursor.rewind();  // set cursor to first chord/rest

    for(i = 0; i<source_measure; i++) source_cursor.nextMeasure();	    // go to specified measure
    while(source_cursor.isRest()) source_cursor.next();					// go to first note in the measure
    var i = 0;
    while(i<source_note_offset){										// skip desired amount of (non-rest) notes
    		if(!source_cursor.isRest()) i++;
    		source_cursor.next();
    } 
    while(source_cursor.isRest()) source_cursor.next();					// advance to the resulting note

//   worked formely, causes crashes now as well??

//   var text  = new Text(curScore);
//	text.text = "source is here";//source_cursor.chord().topNote().name;
//	source_cursor.putStaffText(text);
                
	// initialize target cursor
	
    var target_cursor   = new Cursor(curScore);							// exactly the same procedure as for source cursor
    target_cursor.staff = target_staff;
    target_cursor.voice = target_voice;
    target_cursor.rewind();  
    for(i = 0; i<target_measure_start; i++) target_cursor.nextMeasure();
    while(target_cursor.isRest()) target_cursor.next();
    i = 0;
    while(i<target_note_offset){
    		if(!target_cursor.isRest()) i++;
    		target_cursor.next();
    } 
    while(target_cursor.isRest()) target_cursor.next();

//    text  = new Text(curScore);
//	text.text = "target";//target_cursor.chord().topNote().name;
//	target_cursor.putStaffText(text);
	
	// todo: implement limit to repitching procedure (use target_measure_stop here instead)
	while (!source_cursor.eos()) 
	{
		if (source_cursor.isChord()) 
		{ 	
			addNote(target_cursor, 								// CRASH on second note
					source_cursor.chord().topNote().pitch, 
					target_cursor.chord().tickLen);
		}
		target_cursor.next();
		source_cursor.next();			
		
	}
	curScore.endUndo();
}

// taken from "Create Score" plugin,
// not touched, should work
function addNote(cursor, pitch, duration)
{
	var chord     = new Chord();
	chord.tickLen = duration;
	var note      = new Note();
	note.pitch    = pitch;

	chord.addNote(note);
	cursor.add(chord);
	cursor.next();
};

//---------------------------------------------------------
//    menu:  defines were the function will be placed
//           in the MuseScore menu structure
//---------------------------------------------------------

var mscorePlugin = {
      menu: 'Plugins.Pitch up',
      init: init,
      run:  run
      };

mscorePlugin;

      
// todo: react to selections in document
// 	  var selectionCursor = new Cursor(curScore); 
// 	  selectionCursor.goToSelectionStart();
//       selectionCursor.goToSelectionEnd();
//       form.group_target.spin_target_measure_start.value = selectionCursor.
      